@@ -12,13 +12,28 @@ import jsonref
 from jsonpointer import resolve_pointer
 from docutils import nodes
 from docutils.parsers.rst import directives, Directive
-from myst_parser.main import to_docutils
-
+from pathlib import Path
 
 import json
 from collections import OrderedDict
 
 
+try:
+    from myst_parser.main import to_docutils
+except ModuleNotFoundError:
+    from myst_parser.config.main import MdParserConfig
+    from myst_parser.mdit_to_docutils.base import make_document
+    from myst_parser.mdit_to_docutils.sphinx_ import SphinxRenderer
+    from myst_parser.parsers.mdit import create_md_parser
+
+    # to_docutils was removed in myst-parser>=0.18.
+    def to_docutils(text):
+        # Code is similar to MystParser.parse and myst_parser.parsers.docutils_.Parser.parse.
+        parser = create_md_parser(MdParserConfig(), SphinxRenderer)
+        parser.options["document"] = make_document()
+        return parser.render(text)
+
+
 class CustomJsonrefLoader(jsonref.JsonLoader):
     def get_remote_json(self, uri, **kwargs):
         return {}
@@ -32,6 +47,9 @@ class JSONSchemaDirective(Directive):
         'collapse': directives.unchanged,
         'pointer': directives.unchanged,
         'nocrossref': directives.flag,
+        'addtargets': directives.flag,
+        'externallinks': directives.unchanged,
+        'allowexternalrefs': directives.flag,
     }
     # Add a rollup option here
 
@@ -41,6 +59,7 @@ class JSONSchemaDirective(Directive):
     include_used = set()
     collapse = []
     collapse_used = set()
+    external_links = {}
 
     def run(self):
         include = self.options.get('include')
@@ -52,6 +71,10 @@ class JSONSchemaDirective(Directive):
             self.collapse = collapse.split(',')
             self.collapse_used = set()
 
+        external_links = self.options.get('externallinks')
+        if external_links:
+            self.external_links = json.loads(external_links)
+
         env = self.state.document.settings.env
         try:
             if self.arguments and self.content:
@@ -65,9 +88,12 @@ class JSONSchemaDirective(Directive):
                                        self.arguments[0])
                 env.note_dependency(relpath)
 
-                schema = JSONSchema.loadfromfile(abspath)
+                schema = JSONSchema.loadfromfile(abspath, allow_external_refs=('allowexternalrefs' in self.options))
             else:
-                schema = JSONSchema.loadfromfile(''.join(self.content))
+                schema = JSONSchema.loadfromfile(
+                    ''.join(self.content),
+                    allow_external_refs=('allowexternalrefs' in self.options)
+                )
         except ValueError as exc:
             raise self.error('Failed to parse JSON Schema: %s' % exc)
 
@@ -129,8 +155,10 @@ class JSONSchemaDirective(Directive):
             self.arguments[0].split('/')[-1],
             self.options.get('pointer', ''),
             prop.name)
-        cell = nodes.entry('', nodes.target(ids=[anchor], names=[anchor]), nodes.literal('', nodes.Text(prop.name)),
-                           morecols=1)
+        target = nodes.target(ids=[anchor], names=[anchor.lower()])
+        if 'addtargets' in self.options:
+            self.state.document.note_explicit_target(target)
+        cell = nodes.entry('', target, nodes.literal('', nodes.Text(prop.name)), morecols=1)
         row += cell
         row += self.cell(prop.type)
         row += self.cell(prop.format or '')
@@ -140,7 +168,16 @@ class JSONSchemaDirective(Directive):
         row += self.cell(prop.title)
         if prop.description:
             cell = self.cell(prop.description or '', morecols=3)
-            if 'nocrossref' not in self.options:
+            if prop.name in self.external_links:
+                reference = nodes.reference(
+                    '',
+                    '',
+                    nodes.Text(self.external_links[prop.name]['text']),
+                    internal=False,
+                    refuri=self.external_links[prop.name]['url'],
+                    anchorname='')
+                cell += nodes.paragraph('', nodes.Text('\n\nSee '), reference)
+            elif 'nocrossref' not in self.options:
                 ref = None
                 if hasattr(prop.attributes, '__reference__'):
                     ref = prop.attributes.__reference__['$ref']
@@ -206,8 +243,11 @@ def simplify(obj):
 
 class JSONSchema(object):
     @classmethod
-    def load(cls, reader):
-        obj = jsonref.load(reader, object_pairs_hook=OrderedDict, loader=CustomJsonrefLoader())
+    def load(cls, reader, allow_external_refs=False, base_uri=""):
+        args = {}
+        if not allow_external_refs:
+            args['loader'] = CustomJsonrefLoader()
+        obj = jsonref.load(reader, object_pairs_hook=OrderedDict, base_uri=base_uri, **args)
         return cls.instantiate(None, obj)
 
     @classmethod
@@ -216,9 +256,13 @@ class JSONSchema(object):
         return cls.instantiate(None, obj)
 
     @classmethod
-    def loadfromfile(cls, filename):
+    def loadfromfile(cls, filename, allow_external_refs=False):
         with io.open(filename, 'rt', encoding='utf-8') as reader:
-            return cls.load(reader)
+            args = {}
+            if allow_external_refs:
+                args['allow_external_refs'] = True
+                args['base_uri'] = Path(os.path.realpath(filename)).as_uri()
+            return cls.load(reader, **args)
 
     @classmethod
     def instantiate(cls, name, obj, required=False, parent=None, rollup=True):
@@ -392,7 +436,7 @@ class Array(JSONData):
             item = JSONSchema.instantiate(self.name + '/0', self.items, parent=self)
 
             # array object itself
-            array = Array(self.name, self.attributes, parent=self.parent)
+            array = Array(self.name, self.attributes, required=self.required, parent=self.parent)
             array.type = 'array[%s]' % item.get_typename()
             yield array
 
